from .mockfeatures.features import (
    MockFeatureGroup,
    PhoneBatteryChargingDuration,
    StepCountPerDay,
)

from .mockfeatures.features2 import (
    MockFeatureGroup2,
    StepCountPerHour
)